#pragma once

#include <string>
#include <vector>
#include "CommonType.h"
#include "GTCommonComponentDefine.h"

#define WIDTHBYTES(bits) ((DWORD)(((bits)+31) & (~31)) / 8)
#define DIBWIDTHBYTES(bi) (DWORD)WIDTHBYTES((DWORD)(bi).biWidth * (DWORD)(bi).biBitCount)
#define _DIBSIZE(bi) (DIBWIDTHBYTES(bi) * (DWORD)(bi).biHeight)
#define DIBSIZE(bi) ((bi).biHeight < 0 ? (-1)*(_DIBSIZE(bi)) : _DIBSIZE(bi))

#if defined(WIN32)
namespace RegistryUtil
{
	int ReadRegistry(LPCTSTR szCompany, LPCTSTR szAppName, LPCTSTR szSection, LPCTSTR szDataName, void * pDataValue, int nMaxSize, DWORD * pdwType, BOOL bLocalMachine);
};
#endif // WIN32

namespace UserUtil
{
	bool GetIDAndDomain(WSTRING strFullID, WSTRING& strID, WSTRING& strDomain);
	WSTRING GetIDExceptDomain(WSTRING strFullID);
};


namespace FileUtil
{
	WSTRING ExtractFilePath(LPCWSTR strPath);
	WSTRING ExtractFileName(LPCWSTR strName);
	bool ForceDirectory(LPCWSTR szPath);
};

//namespace ImageUtil
//{
//    bool GetImageCRCFromString(const char *strCRCString, DWORD &dwImgCRC);
//    
//#if defined(WIN32)
//	HBITMAP LoadImage(LPCWSTR pszFileName, bool bDibSection = false, void ** ppDIBits = NULL);
//	bool ResizeImage(const BYTE* pSrcBuffer, // <IN>
//					 int nSrcWidth, 
//					 int nSrcHeight, 
//					 int nSrcBytesPerPixel, 
//					 BYTE* pDestBuffer,		// <OUT>
//					 int nDestWidth, 
//					 int nDestHeight, 
//					 int nDestBytesPerPixel, 
//					 bool bInterpolation = true);
//	void SmoothResize(const BYTE * image_in,  // <IN>
//		 			  int width_in, 
//					  int height_in, 
//					  int byteperpixel_in, 
//					  BYTE * image_out,		 // <OUT>
//					  int width_out, 
//					  int height_out, 
//					  int byteperpixel_out);
//
//#endif
//};

namespace ConvertUtil
{
#if defined(WIN32)	
	int Text2Binary(const char * pText, int nSize, void * pOutBuffer);
#endif
	bool ConvertPhoneStyle(WSTRING &strResult, LPCTSTR szInput);
	
	COMMON_OS_TYPE ConvertToInternalOSType(GTOSType osType);
	GTOSType ConvertToAPIOSType(COMMON_OS_TYPE osType);
    
    GTAVUsageMode ConvertToAPIAVUsageMode(AV_USAGE_MODE avUsageMode);
    AV_USAGE_MODE ConvertToInternalAVUsageMode(GTAVUsageMode avUsageMode);
};


namespace StringUtil
{
#if defined(ANDROID)	// KEH_ANDROID_UNICODE
#else
	inline size_t StrLen(LPCWSTR str)
	{
		return str ? wcslen(str) : 0;
	}
#endif
	inline size_t StrLen(LPCSTR str)
	{
		return str ? strlen(str) : 0;
	}

#if defined(ANDROID)	// KEH_ANDROID_UNICODE
#else
	inline bool IsEmpty(LPCWSTR str)
	{
		return StrLen(str) == 0 ? true : false;
	}
#endif
	inline bool IsEmpty(LPCSTR str)
	{
		return StrLen(str) == 0 ? true : false;
	}

	inline int StrCompare(LPCWSTR str1, LPCWSTR str2)
	{
		return (str1 && str2) ? _wcsicmp(str1, str2) : -1;
	}

	inline int StrCompareA(LPCSTR str1, LPCSTR str2)
	{
		return (str1 && str2) ? _stricmp(str1, str2) : -1;
	}

#if defined(WIN32)
	template <class T>
	void StringFormat(T &strResult, TCHAR *pszFormat, ...)
	{
		TCHAR *pBuffer = NULL;
		va_list argList;
		va_start( argList, pszFormat );
		int nLength = _vscwprintf( pszFormat, argList );
		if(nLength <= 0)
			return;
		pBuffer = new TCHAR[nLength+1];
		if (NULL == pBuffer)
			return;

		_vsnwprintf(pBuffer, nLength, pszFormat, argList );
		pBuffer[nLength] = '\0';
		va_end( argList );

		strResult = pBuffer;
		delete []pBuffer;
		return;
	}
#else
	template <class T>
	void StringFormat(T &strResult, char *pszFormat, ...)
	{
		char *pBuffer = NULL;
		va_list argList;
		va_start( argList, pszFormat );
		int nLength = vsnprintf( NULL, 0, pszFormat, argList );
		if(nLength <= 0)
			return;
		pBuffer = new char[nLength+1];
		vsnprintf( pBuffer, nLength, pszFormat, argList );
		pBuffer[nLength] = '\0';
		va_end( argList );
		
		strResult = pBuffer;
		delete []pBuffer;
		return;
	}
#endif
	
	void StringFormatA(std::string &strResult, char *pszFormat, ...);
	void StringFormatW(WSTRING &strResult, const WCHAR *pszFormat, ...);

	template <class T>
	int StringReplaceAll(T &strSrc, const T& strFind, const T& strReplace)
	{
		int nReplaceCnt = 0;
		size_t offset = 0;  
		while( true )   
		{   
			offset = strSrc.find(strFind, offset);
			if( T::npos == offset )
				break;   
		
			strSrc.replace( offset, strFind.length(), strReplace ); 
			nReplaceCnt++;
		}   
		return nReplaceCnt;
	}

	template <class T>
	bool StringTokenizer(T strSrc, const T& strDelimiter, std::vector<T> &strList)
	{
		size_t offsetPrev = 0;
		size_t offsetNext = 0;
		while( true )   
		{   
			offsetNext = strSrc.find(strDelimiter, offsetPrev);
			if(T::npos == offsetNext )
			{
				if(offsetPrev != 0)
				{
					T sub = strSrc.substr(offsetPrev);
					strList.push_back(sub);
				}
				break;
			}

			T sub = strSrc.substr(offsetPrev, offsetNext - offsetPrev);
			strList.push_back(sub);

			offsetPrev = offsetNext + strDelimiter.length();
		}

		return strList.size() ? true : false;
	}

	template <class T>
	bool StringTokenizer(T strSrc, const T& strDelimiter1, const T& strDelimiter2, std::vector<T> &strList)
	{
		size_t offsetPrev = 0;
		size_t offsetNext = 0;
		while( true )   
		{   
			offsetNext = strSrc.find(strDelimiter1, offsetPrev);
			if(T::npos == offsetNext )
			{
				break;
			}
			offsetNext = strSrc.find(strDelimiter2, offsetNext + strDelimiter1.length());
			if(T::npos == offsetNext )
			{
				break;
			}

			T sub = strSrc.substr(offsetPrev + strDelimiter1.length(), offsetNext - offsetPrev - strDelimiter1.length());
			strList.push_back(sub);

			offsetPrev = offsetNext + strDelimiter2.length();
		}

		return strList.size() ? true : false;
	}
	
	
	// trim from start
	std::string &ltrim(std::string &s);
	
	// trim from end
	std::string &rtrim(std::string &s); 

	// trim from both ends
	std::string &trim(std::string &s);
	
	int wtoi(const WCHAR *str);
	void itoa(int value, std::string& buf, int base);	
	void itow(int value, WSTRING& buf, int base);
};


namespace ParseUtil
{
    GTPenaltyReasonType GetPenaltyReasonFromNumber(int nReason);
	GTPenaltyStatusType GetPenaltyStatusFromNumber(int nStatus);
};

namespace NetworkUtil
{
	bool IPAddressSplit(LPCSTR pszServerAddr, std::string &strAddress, int &nPort);
}

namespace Conversion
{
	const std::string WideToUtf8(const std::wstring& wideString);
	const std::wstring Utf8ToWide(const std::string& utf8String);
	//const std::wstring AcpToWide(const std::string& utf8String);
	//const std::string WideToAcp(const std::wstring& wideString);
	
	char *URLdecode(char *str);
	char *URLencode(char *str);
};